/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.IQ.Disco
 *
 *
 */
ML.load("utils/observable.js");
ML.load("xml/xmldomtraverseutils.js");

/**
 * IQDisco
 *   "JEP0030 - Service Discovery" implementation
 *
 * Constructor:
 *   IQDisco(jabberService)
 *
 * Methods:
 *   getItems(from, node, id)
 *   setItems(to, to_node, jid, node, name, action)
 *   getInfo(from, node, id)
 *
 * Events:
 *   onIQDiscoError(errorCode, errorText, id, from, tag)
 *   onIQDiscoItems(id, from, node, query)
 *   onIQDiscoInfo(id, from, node, query)
 */

function IQDisco(jabberService)
{
    this.base = Observable;
    this.base();

    this.jabberService = jabberService;
}

IQDisco.prototype =
{
    INFO_NS: "http://jabber.org/protocol/disco#info",
    ITEMS_NS: "http://jabber.org/protocol/disco#items",
    __proto__: Observable.prototype,
    namespaceURI: [IQDisco.prototype.INFO_NS, IQDisco.prototype.ITEMS_NS],
    suportedFeatures: {"http://jabber.org/protocol/disco#info": 1},

    getItems: function (jid, node, id)
    {
        return this.jabberService.core.iqTag(null, jid, null, "get", 
					     DB.createElement(this.ITEMS_NS, "query",
							      "node", node),
					     this);
    },
		   
    setItems: function (to, to_node, jid, node, name, action)
    {
	var query = DB.createElement(this.ITEMS_NS, "query",
				     "node", to_node);
	query.addElement(this.ITEMS_NS, "item",
			 "jid", jid,
			 "name", name,
			 "node", node,
			 "action", action);

        return this.jabberService.core.iqTag(null, to, null, "set",
					     query, this);
    },

    getInfo: function (jid, node)
    {
        return this.jabberService.core.iqTag(null, jid, null, "get", 
					     DB.createElement(this.INFO_NS, "query",
							      "node", node), this);
    },

    registerFeature: function(feature)
    {
	this.suportedFeatures[feature] = 1;
    },

    unregisterFeature: function(feature)
    {
	delete this.suportedFeatures[feature];
    },

    onIQResponse: function(tag)
    {
	var queryTag = tag.getElementsByTagName("query").item(0);
	var query = { };

        if (tag.getAttribute("type") == "error") {
            var errTag = tag.getElementsByTagName("error").item(0);

            this.fireEvent("onIQDiscoError",
			   errTag.getAttribute("code"),
			   domToText(errTag),
			   tag.getAttribute("id"),
			   tag.getAttribute("from"),
			   tag);
        } else if (tag.getAttribute("type") == "result" && queryTag)
	    if (queryTag.getAttribute("xmlns") == this.ITEMS_NS) {
		xmlDOMTraverse(queryTag,
			       this.itemsTree,
			       query,
			       "query");

		this.fireEvent("onIQDiscoItems",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"),
			       query.node,
			       query);
	    } else if (queryTag.getAttribute("xmlns") == this.INFO_NS) {
		xmlDOMTraverse(queryTag,
			       this.infoTree,
			       query,
			       "query");

		this.fireEvent("onIQDiscoInfo",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"),
			       query.node,
			       query);
	    } else {
		this.jabberService.core.iqError(tag.getAttribute("to"),
						tag.getAttribute("from"),
						tag.getAttribute("id"),
						501,
						"Not Implemented");
	    }
    },

    onIQHandler: function(tag)
    {
	var queryTag = tag.getElementsByTagName("query").item(0);

	if (queryTag.getAttribute("xmlns") == this.INFO_NS) {
	    var query = DB.createElement(this.INFO_NS, "query");
	    var i;

	    query.addElement(this.INFO_NS, "identity",
			     "category", "user",
			     "type", "client",
			     "name", "JIM");

	    for (i in this.suportedFeatures)
		query.addElement(this.INFO_NS, "feature",
				 "var", i);

	    this.jabberService.core.iqTag(null,
					  tag.getAttribute("from"),
					  tag.getAttribute("id"),
					  "result",
					  query);
	} else {
            this.jabberService.core.iqError(tag.getAttribute("to"),
					    tag.getAttribute("from"),
					    tag.getAttribute("id"),
					    501,
					    "Not Implemented");
	}
    },

    itemsTree: {
	__ATTRIBUTES__: {
	    "node": new XMLDOMAttrActionSetProp(),
	},
	"item": {
	    __PROCESS__: new XMLDOMActionRecursive("items"),
	    __ATTRIBUTES__: {
		"jid": new XMLDOMAttrActionSetProp(),
		"name": new XMLDOMAttrActionSetProp(),
		"node": new XMLDOMAttrActionSetProp(),
		"action": new XMLDOMAttrActionSetProp(),
	    }
	}
    },
    infoTree: {
	__ATTRIBUTES__: {
	    "node": new XMLDOMAttrActionSetProp(),
	},
	"identity": {
	    __PROCESS__: new XMLDOMActionRecursive("identities"),
	    __ATTRIBUTES__: {
		"category": new XMLDOMAttrActionSetProp(),
		"name": new XMLDOMAttrActionSetProp(),
		"type": new XMLDOMAttrActionSetProp(),
	    }
	},
	"feature": {
	    __PROCESS__: new XMLDOMActionRecursive("features"),
	    __ATTRIBUTES__: {
		"var": new XMLDOMAttrActionSetProp(),
	    }
	}
    }
}
