/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.IQ.FeatNeg
 *
 *
 */
ML.load("utils/observable.js");
ML.load("services/x/data.js");
ML.load("xml/xmldomtraverseutils.js");

/**
 * IQFeatNeg
 *   "JEP0020 - Feature Negotiation" implementation
 *
 * Constructor:
 *   IQFeatNeg(jabberService)
 *
 * Methods:
 *   getOptions(from, feature);
 *   chooseOption(to, feature, optionName)
 *   chooseOptionData(to, feature, xdata)
 *   sendOptionsData(to, id, feature, xdata)
 *   sendResult(to, id)
 *   sendError(to, id, code, desc)
 *   registerFeature(feature, handler)
 *   unregisterFeature(feature)
 *
 * Events:
 *   onIQFeatNegError(errorCode, errorText, id, from, tag)
 *   onIQFeatNegOptions(id, from, xdata)
 *   onIQFeatNegResult(id, from)
 *
 */

function IQFeatNeg(jabberService)
{
    this.base = Observable;
    this.base();

    this.jabberService = jabberService;
}

IQFeatNeg.prototype =
{
    FEATNEG_NS: "http://jabber.org/protocol/feature-neg",
    __proto__: Observable.prototype,
    namespaceURI: IQFeatNeg.prototype.FEATNEG_NS,
    features: {},

    getOptions: function (from, feature)
    {
	var query = DB.createElement(this.FEATNEG_NS, "query");
	var xdata = new XData(xdata.result = {
				  "type": "submit",
				  "fields": {
				      "var": feature
				  }
			      });

	xdata.createDOM(query);

        return this.jabberService.core.iqTag(null, from, null, "get",
					     query, this);
    },

    chooseOptions: function (from, feature, option)
    {
	var query = DB.createElement(this.FEATNEG_NS, "query");
	var xdata = new XData(xdata.result = {
				  "type": "submit",
				  "fields": {
				      "var": feature,
				      "values": [option]
				  }
			      });

	xdata.createDOM(query);

        return this.jabberService.core.iqTag(null, from, null, "set",
					     query, this);
    },

    chooseOptionsData: function (from, feature, xdata)
    {
	var query = DB.createElement(this.FEATNEG_NS, "query");

	xdata.createDOM(query);
        return this.jabberService.core.iqTag(null, from, null, "set",
					     query, this);
    },

    sendOptionsData: function (to, id, feature, xdata)
    {
	var query = DB.createElement(this.FEATNEG_NS, "query");

	xdata.createDOM(query);
        return this.jabberService.core.iqTag(null, to, id, "result", query);
    },

    sendResult: function (to, id)
    {
        return this.jabberService.core.iqTag(null, to, id, "result");
    },

    sendError: function (to, id, code, desc)
    {
        return this.jabberService.core.iqError(null, to, id, code, desc);
    },

    registerFeature: function(feature, handler)
    {
	this.features[feature] = handler;
    },

    unregisterFeature: function(feature)
    {
	delete this.features[feature];
    },

    onIQResponse: function(tag)
    {
	var queryTag;

        if (tag.getAttribute("type") == "error") {
            var errTag = tag.getElementsByTagName("error").item(0);

            this.fireEvent("onIQFeatNegError",
			   errTag.getAttribute("code"),
			   domToText(errTag),
			   tag.getAttribute("id"),
			   tag.getAttribute("from"),
			   tag);
        } else if (tag.getAttribute("type") == "result")
	    if (queryTag = tag.getElementsByTagName("query").item(0))
		this.fireEvent("onIQFeatNegOptions",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"),
			       new XData(queryTag.
					 getElementsByTagName("x").items(0)));
	    else
		this.fireEvent("onIQFeatNegResult",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"));
	else
	    this.jabberService.core.iqError(tag.getAttribute("to"),
					    tag.getAttribute("from"),
					    tag.getAttribute("id"),
					    501,
					    "Not Implemented");
    },

    onIQHandler: function(tag)
    {
	var queryTag = tag.getElementsByTagName("query").item(0);
	var xdata;
	var handler;
	var type;

	if (queryTag) {
	    xdata = new XData(queryTag.
			      getElementsByTagName("x").items(0));
	    if (xdata.result.fields && xdata.result.fields[0]) {
		handler = this.features[xdata.fields[0]["var"]];

		if ((type = tag.getAttribute("type")) == "get")
		    handler.onIQFeatNegHandlerGet(tag.getAttribute("id"),
						  tag.getAttribute("from"),
						  xdata);
		else if (type == "set")
		    handler.onIQFeatNegHandlerSet(tag.getAttribute("id"),
						  tag.getAttribute("from"),
						  xdata);
		else
		    this.jabberService.core.iqError(tag.getAttribute("to"),
						    tag.getAttribute("from"),
						    tag.getAttribute("id"),
						    501,
						    "Not Implemented");
	    }
	} else
	    this.jabberService.core.iqError(tag.getAttribute("to"),
					    tag.getAttribute("from"),
					    tag.getAttribute("id"),
					    501,
					    "Not Implemented");
    }
}
